/** Implementation for NSComparisonPredicate for GNUstep
   Copyright (C) 2005 Free Software Foundation, Inc.

   Written by:  Quentin Mathe <qmathe@club-internet.fr>
   Date: May 2005

   This file is part of the GNUstep Base Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.

   */

#include <Foundation/NSError.h>
#include <Foundation/NSException.h>
#include <Foundation/NSObjCRuntime.h>
#include <Foundation/NSExpression.h>
#include <Foundation/NSComparisonPredicate.h>


@interface NSComparisonPredicate (GNUstepPrivate)
- (id) _initWithLeftExpression: (NSExpression *)lhs 
               rightExpression: (NSExpression *)rhs;
@end


@implementation	NSComparisonPredicate

/*
 * Factory methods
 */
 
+ (NSPredicate *) predicateWithLeftExpression: (NSExpression *)lhs 
                                        rightExpression: (NSExpression *)rhs
                                         customSelector: (SEL)selector
{
  NSPredicate *predicate = [[NSComparisonPredicate alloc]
    initWithLeftExpression: lhs rightExpression: rhs customSelector: selector];

  return AUTORELEASE(predicate);
}

+ (NSPredicate *) predicateWithLeftExpression: (NSExpression *)lhs 
                              rightExpression: (NSExpression *)rhs 
                                     modifier: (NSComparisonPredicateModifier)modifier 
                                         type: (NSPredicateOperatorType)type 
                                      options: (unsigned)options
{
  NSPredicate *predicate = 
   [[NSComparisonPredicate alloc] initWithLeftExpression: lhs
   rightExpression: rhs modifier: modifier type: type options: options];

  return AUTORELEASE(predicate);
}

/*
 * Other methods
 */

- (NSComparisonPredicateModifier) comparisonPredicateModifier
{
  return _predicateModifier;
}

- (SEL) customSelector
{
  return _customSelector;
}

/*
 * Init methods
 */
 
- (NSPredicate *) initWithLeftExpression: (NSExpression *)lhs 
                         rightExpression: (NSExpression *)rhs 
                          customSelector: (SEL)selector
{
  self = [self _initWithLeftExpression: lhs rightExpression: rhs]; 

  _customSelector = selector;

  return self;
}
                         
- (id) initWithLeftExpression: (NSExpression *)lhs 
              rightExpression: (NSExpression *)rhs 
                     modifier: (NSComparisonPredicateModifier)modifier 
                         type: (NSPredicateOperatorType)type 
                      options: (unsigned)options
{
  self = [self _initWithLeftExpression: lhs rightExpression: rhs]; 

  _predicateModifier = modifier;
  _operatorType = type;
  _options = options;

  return self;
}

- (void) dealloc
{
  RELEASE(_lhs);
  RELEASE(_rhs);

  [super dealloc];
}

/*
 * Other methods
 */

- (NSExpression *) leftExpression
{
  return _lhs;
}

- (unsigned) options
{
  return _options;
}

- (NSPredicateOperatorType) predicateOperatorType
{
  return _operatorType;
}

- (NSExpression *) rightExpression
{
  return _rhs;
}

/*
 * Overriden methods
 */

- (BOOL) evaluateWithObject: (id)object
{
  id leftValue = nil;
  id rightValue = nil;
  BOOL value = NO;
  NSMutableDictionary *context = [NSMutableDictionary dictionary];

  leftValue = [_lhs expressionValueWithObject: object context: context];
  rightValue = [_rhs expressionValueWithObject: object context: context];

  switch (_operatorType)
  {
    case NSLessThanOrEqualToPredicateOperatorType:
      break;
    case NSGreaterThanPredicateOperatorType:
      break;
    case NSGreaterThanOrEqualToPredicateOperatorType:
      break;
    case NSEqualToPredicateOperatorType:
      value = [leftValue isEqual: rightValue];
      break;
    case NSNotEqualToPredicateOperatorType:
      break;
    case NSMatchesPredicateOperatorType:
      break;
    case NSLikePredicateOperatorType:
      break;
    case NSBeginsWithPredicateOperatorType:
      break;
    case NSEndsWithPredicateOperatorType:
      break;
    case NSInPredicateOperatorType:
      break;
    case NSCustomSelectorPredicateOperatorType:
      value = 
        [leftValue performSelector: _customSelector withObject: rightValue];
      break;
  }

  return value;
}

@end

@implementation NSComparisonPredicate (GNUstepPrivate)

- (id) _initWithLeftExpression: (NSExpression *)lhs 
               rightExpression: (NSExpression *)rhs
{
  if ((self = [super init]) != nil)
  {
    ASSIGN(_lhs, lhs);
    ASSIGN(_rhs, rhs);

    return self;
  }

  return nil;
}

@end
