/** Implementation for NSCompoundPredicate for GNUstep
   Copyright (C) 2005 Free Software Foundation, Inc.

   Written by:  Quentin Mathe <qmathe@club-internet.fr>
   Date: May 2005

   This file is part of the GNUstep Base Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.

   */

#include <Foundation/NSArray.h>
#include <Foundation/NSError.h>
#include <Foundation/NSException.h>
#include <Foundation/NSObjCRuntime.h>
#include <Foundation/NSCompoundPredicate.h>


@implementation	NSCompoundPredicate

/*
 * Factory methods
 */

/** Returns a new predicate formed by AND-ing the predicates specified by subpredicates. */ 
+ (NSPredicate *) andPredicateWithSubpredicates: (NSArray *)subpredicates
{
  NSPredicate *p = [[NSCompoundPredicate alloc] 
    initWithType: NSAndPredicateType subpredicates: subpredicates];
  
  return AUTORELEASE(p);
}

/** Returns a new predicate formed by NOT-ing the predicate specified by predicate. */
+ (NSPredicate *) notPredicateWithSubpredicate: (NSPredicate *)subpredicate
{
  NSPredicate *p = [[NSCompoundPredicate alloc] initWithType: NSNotPredicateType 
    subpredicates: [NSArray arrayWithObject: subpredicate]];
  
  return AUTORELEASE(p);
}

/** Returns a new predicate formed by OR-ing the predicates specified by subpredicates. */
+ (NSPredicate *) orPredicateWithSubpredicates: (NSArray *)subpredicates
{
  NSPredicate *p = [[NSCompoundPredicate alloc] 
    initWithType: NSOrPredicateType subpredicates: subpredicates];
  
  return AUTORELEASE(p);
}

/*
 * Other methods
 */

/** Returns the predicate type for the receiver. */
- (NSCompoundPredicateType) compoundPredicateType
{
  return _predicateType;
}

/*
 * Init methods
 */

/** Initializes the receiver, setting its type to type and subpredicates array to subpredicates. */
- (id) initWithType: (NSCompoundPredicateType)type 
     subpredicates: (NSArray *)subpredicates
{
  if ((self = [super init]) != nil)
  {
  	_predicateType = type;
  	_subpredicates = RETAIN(subpredicates);
  	
  	return self;
  }
  
  return nil;
}

- (void) dealloc
{
  RELEASE(_subpredicates);
  [super dealloc];
}
     
/*
 * Other methods
 */

/** Returns the array of the receiver subpredicates. */
- (NSArray *) subpredicates
{
  return _subpredicates;
}

/*
 * Overriden methods
 */

- (BOOL) evaluateWithObject: (id)object
{
  BOOL predicateValue = NO;
  BOOL value = NO;
  NSEnumerator *e = [_subpredicates objectEnumerator];
  NSPredicate *p = nil;

  while ((p = [e nextObject]) != nil)
  {
    predicateValue = [p evaluateWithObject: object];

    switch (_predicateType)
    {
      case NSAndPredicateType:
        value = value && predicateValue;
        break;
      case NSOrPredicateType:
        value = value || predicateValue;
        break;
      case NSNotPredicateType:
        value = !predicateValue;
        return value; /* We should have only one subpredicate here */
    }

  }

  return value;
}

@end
