/** Implementation for NSExpression for GNUstep
   Copyright (C) 2005 Free Software Foundation, Inc.

   Written by:  Quentin Mathe <qmathe@club-internet.fr>
   Date: May 2005

   This file is part of the GNUstep Base Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.

   */

#include <Foundation/NSArray.h>
#include <Foundation/NSDictionary.h>
#include <Foundation/NSError.h>
#include <Foundation/NSException.h>
#include <Foundation/NSValue.h>
#include <Foundation/NSObjCRuntime.h>
#include <Foundation/NSString.h>
#include <Foundation/NSExpression.h>

static NSArray *allowedFunctionNames = nil;

@interface NSExpression (GNUstepPrivate)
+ (NSExpression *) _expressionWithBooleanValue: (BOOL)value;
- (NSNumber *) _evaluateFunctionWithArguments;
@end


@implementation	NSExpression

+ (void) initialize
{
  if (self == [NSExpression class])
  {
    allowedFunctionNames = 
     [NSArray arrayWithObjects: @"avg", @"count", @"max", @"min", @"sum", nil];
  }
}

/*
 * Factory methods
 */

/** Returns a new expression that represents a constant value. */
+ (NSExpression *)expressionForConstantValue: (id)obj
{
  NSExpression *expr =
    [[NSExpression alloc] initWithExpressionType: NSConstantValueExpressionType];
  
  expr->_exprConstantValue = RETAIN(obj);
  
  return AUTORELEASE(expr);
}

/* Returns a new expression that represents the object being evaluated. */
+ (NSExpression *)expressionForEvaluatedObject
{
  NSExpression *expr =
    [[NSExpression alloc] initWithExpressionType: NSEvaluatedObjectExpressionType];
  
  expr->_exprOperand = nil;
  
  return AUTORELEASE(expr);
}

/** Returns a new expression that invokes a predefined function. The name 
    parameter can be one of the following predefined functions.
    This method raises an exception immediately if the selector is invalid; it
    raises an exception at runtime if the parameters are incorrect. */
+ (NSExpression *)expressionForFunction: (NSString *)name 
                              arguments: (NSArray *)parameters
{
  NSExpression *expr =
    [[NSExpression alloc] initWithExpressionType: NSFunctionExpressionType];
  NSEnumerator *e = [parameters objectEnumerator];
  id parameter;
  
  /* Check name is bound to a valid function. */
  if ([allowedFunctionNames containsObject: name] == NO)
  {
     // FIXME: An exception should be thrown here probably.
     NSLog(@"Function %@ isn't a valid function name.", name);
    	  
     return nil;
  }
  
  /* Check parameters are NSExpression instances. */
  while ((parameter = [e nextObject]) != nil)
  {
    Class parameterClass = [parameter class];
    
    if ([parameterClass isEqual: [NSExpression class]] == NO)
    {
    	// FIXME: An exception should be thrown here probably.
    	NSLog(@"Argument %@ is an instance of class %@ and not of class \
          NSExpression.", parameter, parameterClass);
    	  
    	return nil;
    }
  }
  
  expr->_functionName = RETAIN(name);
  expr->_functionArguments = RETAIN(parameters);
  
  return AUTORELEASE(expr);
}

/** Returns a new expression that invokes valueForKeyPath: with keyPath. */
+ (NSExpression *)expressionForKeyPath: (NSString *)keyPath
{
  NSExpression *expr =
    [[NSExpression alloc] initWithExpressionType: NSKeyPathExpressionType];
    
  expr->_exprValue = RETAIN(keyPath);
  
  return AUTORELEASE(expr);
}

/** Returns a new expression that extracts a value from the variable bindings
    dictionary. */
+ (NSExpression *)expressionForVariable:(NSString *)string
{
  NSExpression *expr = 
    [[NSExpression alloc] initWithExpressionType: NSVariableExpressionType];
  
  expr->_exprValue = RETAIN(string);
  
  return AUTORELEASE(expr);
}

/*
 * Other methods
 */

/** Returns the arguments for the receiver—that is, the array of expressions 
    that will be passed as parameters during invocation of the selector on the
    operand of a function expression. Raises an exception if not applicable. */
- (NSArray *)arguments
{
  if ([self expressionType] != NSFunctionExpressionType)
  {
    [NSException raise: @"InvalidStateException" format: @"Expression %@ is \
      not a function expression", self];
  }
  
  return _functionArguments;
}

/** Returns the constant value for the receiver. Raises an exception if not 
    applicable. */
- (id)constantValue
{
  if ([self expressionType] != NSConstantValueExpressionType)
  {
    [NSException raise: @"InvalidStateException" format: @"Expression %@ is \
      not a constant value expression", self];
  }
  
  return _exprConstantValue;
}

/** Returns the expression type for the receiver. The expression type is an 
    NSExpressionType. Raises an exception if not applicable. **/
- (NSExpressionType)expressionType
{
  return _exprType;
}

/** Evaluates the expression using the specified object and context. Note that 
    context is mutable—it can be used by expressions to store temporary state 
    for one predicate evaluation. */
- (id)expressionValueWithObject: (id)object 
                        context: (NSMutableDictionary *)context
{
  id value = nil;
  
  _exprOperand = object;
  
  switch (_exprType)
  {
    case NSConstantValueExpressionType:
      value = [self constantValue];
      break;
      
    case NSEvaluatedObjectExpressionType:
      value = _exprOperand;
      break;
      
    case NSVariableExpressionType:
      value = [context objectForKey: [self variable]];
      break;
      
    case NSKeyPathExpressionType:
      value = [_exprOperand valueForKeyPath: [self keyPath]];
      break;
      
    case NSFunctionExpressionType:
      value = [self _evaluateFunctionWithArguments];
      break;
      
    default: /* Must never happens */
      NSLog(@"Undefined NSExpression type");
  }
  
  return value;
}

/** Returns the function for the receiver. Raises an exception if not 
    applicable. */
- (NSString *)function
{
  if ([self expressionType] != NSFunctionExpressionType)
  {
    [NSException raise: @"InvalidStateException" format: @"Expression %@ is \
      not a function expression", self];
  }

  return _functionName;
}

/*
 * Init methods
 */

/** Initializes the receiver with the specified expression type. The type 
    parameter represents the expression type as defined by NSExpressionType. 
    This method is the designated initializer for NSExpression. */
- (id)initWithExpressionType: (NSExpressionType)type
{
  if ((self = [super init]) != nil)
  {
    _exprType = type;
  	
  	return self;
  }
  
  return nil;
}

- (void) dealloc
{
  RELEASE(_functionName);
  RELEASE(_functionArguments);
  RELEASE(_exprOperand);
  RELEASE(_exprConstantValue);
  RELEASE(_exprValue);
  
  [super dealloc];
}

/*
 * Other methods
 */

/** Returns the key path for the receiver. Raises an exception if not 
    applicable. */
- (NSString *)keyPath
{
  if ([self expressionType] != NSKeyPathExpressionType)
  {
    [NSException raise: @"InvalidStateException" format: @"Expression %@ is \
      not a key path expression", self];
  }
    
  return _exprValue;
}

/** Returns the operand for the receiver—that is, the object on which the 
    selector will be invoked. The object is the result of evaluating a key path
    or one of the defined functions. Raises an exception if not applicable. */
- (NSExpression *)operand
{  
  if ([self expressionType] != NSKeyPathExpressionType
    || [self expressionType] != NSFunctionExpressionType )
  {
    [NSException raise: @"InvalidStateException" format: @"Expression %@ is \
      not a key path expression or a function expression", self];
  }
  
  return _exprOperand;
}

/** Returns the variable for the receiver. Raises an exception if not applicable. */
- (NSString *)variable
{
  if ([self expressionType] != NSVariableExpressionType)
  {
    [NSException raise: @"InvalidStateException" format: @"Expression %@ is \
      not variable expression", self];
  }
  
  return _exprValue;
}

@end

/*
 * Private methods
 */

@implementation NSExpression (GNUstepPrivate)

+ (NSExpression *) _expressionWithBooleanValue: (BOOL)value
{
  NSExpression *expr = [[NSExpression alloc] initWithExpressionType:
    NSConstantValueExpressionType];

  expr->_exprBoolValue = value;

  return AUTORELEASE(expr);
}

- (NSNumber *) _evaluateFunctionWithArguments
{
  NSString *f = [NSString stringWithFormat: @"@@%.constantValue", _functionName];
  NSNumber *number;
  
  // FIXME: Check exceptions here.
  
  number = [_functionArguments valueForKey: f];
  
  return number;
}

@end
